// CorrelationObserverFFT.cpp: implementation of the CorrelationObserverFFT class.
//
//////////////////////////////////////////////////////////////////////

// Modified from Nick Bailey's version

#include "CorrelationObserverFFT.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
// #define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CorrelationObserverFFT::CorrelationObserverFFT()
{
	width = 0;
	numberAveraged = 0;

	FTreal = NULL;
	FTimag = NULL;
	wsave = NULL;
	correlationFunction = NULL;
}

void CorrelationObserverFFT::Allocate(int width)
{
	if (FTreal)					delete [] FTreal;
	if (FTimag)					delete [] FTimag;
	if (wsave)					delete [] wsave;
	if (correlationFunction)	delete [] correlationFunction;

	this->width = width;
	FTreal = new double[width];
	FTimag = new double[width];
	wsave = new double[3*width];
	correlationFunction = new double[width];

	Reset();
}

CorrelationObserverFFT::~CorrelationObserverFFT()
{
	if (FTreal)					delete [] FTreal;
	if (FTimag)					delete [] FTimag;
	if (wsave)					delete [] wsave;
	if (correlationFunction)	delete [] correlationFunction;
}

void CorrelationObserverFFT::Update(Subject *sim)
{
	// Reset everything to zero each measurement
	// There is enough averaging over rows as it is, at least for coarsening!
	Reset();	

	SpinLattice *S = ((IsingSimulation *) sim)->GetLattice();
	if (width != S->GetWidth())
	{
		width = S->GetWidth();
		Allocate(width);		// Allocate arrays

		// Set up table of coefficients
		zfft1dc(FTreal, FTimag, width, 0, wsave);
	}

//  Fourier transform, absolute square, FT back, yields correlation function

	for (int i=0; i<S->GetHeight(); i++)
	{
		// Fill FTarray with spins
		for (int j=0; j<width; j++)
		{
			FTreal[j] = (double) S->Get(i,j);
			FTimag[j] = 0.0;
		}

		// fft it
		zfft1dc(FTreal, FTimag, width, -1, wsave);

		// Absolute square of the Fourier transform
		for(j=0; j<width; j++)
		{
			FTreal[j] = FTreal[j]*FTreal[j] + FTimag[j]*FTimag[j];
			FTimag[j] = 0.0;
		}
	
		// inverse fft the result--> should be the autocorrelation 

		zfft1dc(FTreal, FTimag, width, 1, wsave);

		// Add it to the correlation function summed so far

		for (j=0; j<width; j++)
			correlationFunction[j] += FTreal[j];

		numberAveraged++;
	}

}

void CorrelationObserverFFT::Reset()
{
	for (int j=0; j<width; j++)
	{
		correlationFunction[j]=0;
	}
	numberAveraged = 0;
}
