// CorrelationObserverSlow.cpp: implementation of the CorrelationObserverSlow class.
//
//////////////////////////////////////////////////////////////////////

#include "CorrelationObserverSlow.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
// #define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CorrelationObserverSlow::CorrelationObserverSlow()
{
	width = 0;
	numberAveraged = 0;
	correlationFunction = NULL;
}

CorrelationObserverSlow::~CorrelationObserverSlow()
{
	if (correlationFunction != NULL)
		delete[] correlationFunction;
}

void CorrelationObserverSlow::Update(Subject *sim)
{
	// Reset everything to zero each measurement
	// There is enough averaging over rows as it is, at least for coarsening!
	Reset();	

	// Does slow version of correlation function along columns j of spin lattice
	SpinLattice *S = ((IsingSimulation *) sim)->GetLattice();

	if (width != S->GetWidth())
	{
		width = S->GetWidth();
		if (correlationFunction != NULL)
			delete [] correlationFunction;
		correlationFunction = new double[S->GetWidth()];
		Reset();
	}

//  Very slow! Goes as L^3! Should
	// Do selected columns, or
	// Use Fourier transforms (L^2 log L)

// This is only along x-axis: should do 2-D correlation functions

	for (int i=0; i<S->GetHeight(); i++)
	{
		for (int j=0; j<width; j++)
		{
			for (int r=0; r<=j; r++)
			{
				correlationFunction[r] += S->Get(i,j)*S->Get(i,j-r);
			}
		}
		numberAveraged++;
	}

}

void CorrelationObserverSlow::Reset()
{
	for (int j=0; j<width; j++)
	{
		correlationFunction[j]=0;
	}
	numberAveraged = 0;
}
