// IsingSimulation.cpp: implementation of the IsingSimulation class.
//
//////////////////////////////////////////////////////////////////////

#include "IsingSimulation.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
// XXX #define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IsingSimulation::IsingSimulation()
{
	S = new SpinLattice(128,128);		// XXX
	S->SetRandom(1);
//	spinDynamics = new SpinDynamics(S);		// Heat bath
//	spinDynamics = new WolffDynamics(S);	// Wolff Dynamics
	spinDynamics = new ContinuousTime(S);	// Continuous Time Dynamics

	SetMagneticField(0.0);
	SetTemperature(1.0);
	SetTime(0.0);

}

IsingSimulation::~IsingSimulation()
{
	delete S;
	delete spinDynamics;
}

SpinLattice * IsingSimulation::GetLattice()		// No const! Allow people outside to reset spins [for now!]
{
	return S;
}

void IsingSimulation::SetHeightWidth(int height, int width)
{
	if (S->GetWidth() != width || S->GetHeight() != height)
	{
		delete S;
		S = new SpinLattice(height,width);
		S->SetRandom(1);
		spinDynamics->ResetLattice(S);
	}
}

double IsingSimulation::Sweep()
{
	double deltaTime = 0.0;
	while (deltaTime < 1.0)
	{
		deltaTime += spinDynamics->Sweep();
	}

	time += deltaTime;

	return time;
}

void IsingSimulation::SetTemperature(double temperature)
{
	this->temperature = temperature;
	spinDynamics->SetTemperature(temperature);
}

void IsingSimulation::SetMagneticField(double magneticField)
{
	this->magneticField = magneticField;
	spinDynamics->SetMagneticField(magneticField);
}

SpinDynamics * IsingSimulation::SwitchDynamics(SpinDynamics *spinDynamics)
{
	SpinDynamics * oldDynamics = this->spinDynamics;
	this->spinDynamics = spinDynamics;
	spinDynamics->SetTemperature(temperature);
	spinDynamics->SetMagneticField(magneticField);

	spinDynamics->ResetLattice(S);
	return oldDynamics;
}
