// IsingSimulation.h: interface for the IsingSimulation class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ISINGSIMULATION_H
#define ISINGSIMULATION_H

#ifndef SWIG
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
#endif // SWIG

#include "SpinLattice.h"
#include "SpinDynamics.h"
#include "SubjectObserver.h"
#include "WolffDynamics.h"
#include "ContinuousTime.h"

class IsingSimulation : public Subject
{
public:
	SpinDynamics * SwitchDynamics(SpinDynamics *spinDynamics);
	void SetMagneticField(double magneticField);
	double GetMagneticField() const {return magneticField;}
	void SetTemperature(double temperature);
	double GetTemperature() const {return temperature;}
	int GetWidth() const {return S->GetWidth();}
	int GetHeight() const {return S->GetHeight();}
	void SetTime(double time) {this->time = time;}
	double GetTime() const {return time;}
	double Sweep();
	void SetHeightWidth(int height, int width);
	SpinLattice* GetLattice();
	SpinDynamics* GetDynamics() const {return spinDynamics;}
	IsingSimulation();
	virtual ~IsingSimulation();

protected:
	double magneticField;
	double temperature;
	SpinDynamics * spinDynamics;
	SpinLattice * S;
	double time;
};

#endif // !defined(ISINGSIMULATION_H)
