// SpinDynamics.cpp: implementation of the SpinDynamics class.
//
//////////////////////////////////////////////////////////////////////

#include "SpinDynamics.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SpinDynamics::SpinDynamics(SpinLattice *S)
{
	this->S = S;
	rand.seed(1);
	SetTemperature(1.0);	// Defaults
	SetMagneticField(0.0);	// Defaults
}

SpinDynamics::~SpinDynamics()
{

}

double SpinDynamics::Sweep()
{

	// Start with table of probUps

	double probUp[5];	// Square lattice neighbors up run from 0 to 4

	for (int numUp = 0; numUp <= 4; numUp ++)
	{
		double dE = -2 * (2 * (numUp-2) + magneticField);
		probUp[numUp] = 1./(1.+exp(dE/temperature));
	}

	for (int tries=0; tries < S->GetWidth()*S->GetHeight(); tries++)
	{
		int i = rand.discrete(0,S->GetHeight()-1);
		int j = rand.discrete(0,S->GetWidth()-1);
		
		if (probUp[S->GetNeighborsUp(i,j)] > rand.uniform())
			S->Set(i,j,1);
		else
			S->Set(i,j,-1);
	}

	return 1.;	// One sweep = one MC step per spin

}

void SpinDynamics::SetTemperature(double temperature)
{
	this->temperature=temperature;
	// Could store flip probabilities for different environments
	// at the temperature
}

void SpinDynamics::ResetLattice(SpinLattice * S)
{
	this->S = S;
}

void SpinDynamics::SetMagneticField(double magneticField)
{
	this->magneticField = magneticField;
	// Store flip probabilities for different environments
	// at the field and temperature
}


void SpinDynamics::SetRadius(double radius)
{
	int centerI = S->GetHeight()/2;
	int centerJ = S->GetWidth()/2;
	double radius2 = radius*radius;

	for (int i=0; i<S->GetHeight(); i++)
	{
		for (int j=0; j<S->GetWidth(); j++)
		{
			int r2 = (i-centerI)*(i-centerI) + (j-centerJ)*(j-centerJ);
			if (r2 < radius2) S->Set(i,j,1);
			else S->Set(i,j,-1);
		}
	}

	ResetLattice(S);

}
