// SpinLattice.cpp: implementation of the SpinLattice class.
//
//////////////////////////////////////////////////////////////////////

#include "SpinLattice.h"
#include "rand.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
// XXX #define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


SpinLattice::~SpinLattice()
{
	delete [] S[0];	// Constructed as one large array: delete once
	delete [] S;	// Delete pointers into array
}

void SpinLattice::GenerateLattice(int height, int width)
{
	S = new spin* [height];
	S[0] = new spin [height*width];	// Construct as one array, so memory in one unit
	for (int k=1; k<height; k++)
	{
//		S[k] = S[0] + k*width;	This should work too!
		S[k] = &(S[0][k*width]);	// Pointer to first element in row k
	}
	this->height = height;
	this->width = width;
	SetUp();
}

void SpinLattice::SetUp()
{
	magnetization = width*height;
	for (int i=0; i<width*height; i++) S[0][i] = 1;
}

void SpinLattice::SetDown()
{
	magnetization = -width*height;
	for (int i=0; i<width*height; i++) S[0][i] = -1;
}


void SpinLattice::SetRandom(int seed)
{
	Rand rand(seed);	// Initialize random number generator with seed

	magnetization = 0;
	for (int i=0; i<width*height; i++) 
	{
		S[0][i] = 2*rand.discrete(0,1)-1;	// Either -1 or 1
		magnetization += S[0][i];
	}
}

SpinLattice::SpinLattice(int height, int width)
{
	GenerateLattice(height, width);
}
