// SpinLattice.h: interface for the SpinLattice class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SPINLATTICE_H
#define SPINLATTICE_H

#ifndef SWIG
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
#endif // SWIG

#include <assert.h>

typedef char spin;	// char is just one-byte integer; call it spin to avoid confusion?

class SpinLattice  
{
public:
	int GetMagnetization() const {return magnetization;}
	int GetNeighborsUp(int i, int j) const
	{
		int neighborsUp = 0;
		int iNbr = i+1;
		if (iNbr == height) iNbr =0;
		if (S[iNbr][j]==1) neighborsUp++;
		iNbr = i-1;
		if (iNbr == -1) iNbr = height-1;
		if (S[iNbr][j]==1) neighborsUp++;

		int jNbr = j+1;
		if (jNbr == width) jNbr =0;
		if (S[i][jNbr]==1) neighborsUp++;
		jNbr = j-1;
		if (jNbr == -1) jNbr = width-1;
		if (S[i][jNbr]==1) neighborsUp++;
		
		return neighborsUp;
	}
	SpinLattice(int height, int width);
	void SetRandom(int seed);
	void SetDown();
	void SetUp();
	// We could refer to S[i][j] safely by using operator overloading
	// const spin*operator[](int i) {return S[i];}
	// but we'll use S.Get(i,j) just to be simpler
	spin Get(int i, int j) const { return S[i][j]; }
	void Set(int i, int j, spin s) 
	{
		if (s != S[i][j])
		{
			assert(i<height && j<width);
			assert(s==1 || s==-1);
			magnetization += 2*s;
			// Could put in total energy update here
			S[i][j] = s;
		}
	}
	int GetWidth() {return width;}
	int GetHeight() {return height;}
	virtual ~SpinLattice();

protected:
	int magnetization;
	int height;
	int width;
	void GenerateLattice(int height, int width);
	spin ** S;
};

#endif // !defined(SPINLATTICE_H)
