// WolffDynamics.cpp: implementation of the WolffDynamics class.
//
//////////////////////////////////////////////////////////////////////

#include "WolffDynamics.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
// #define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

WolffDynamics::WolffDynamics(SpinLattice *S) : SpinDynamics(S)
{
}

WolffDynamics::~WolffDynamics()
{

}

void WolffDynamics::SetTemperature(double temperature)
{
	this->temperature = temperature;
	if (temperature==0.0 || 2./temperature > 300.) p = 1.;
	else p = 1. - exp(-2./temperature);

}

double WolffDynamics::Sweep()
{
	
	int i = rand.discrete(0,S->GetHeight()-1);
	int j = rand.discrete(0,S->GetWidth()-1);

	oldSpin = S->Get(i,j);

	S->Set(i,j,-oldSpin);

	double spinsFlipped = FlipNeighbors(i,j)+1;	// Double so int divide doesn't return zero

	return spinsFlipped/(S->GetHeight() * S->GetWidth());

}

int WolffDynamics::FlipNeighbors(int i, int j)
{
	int numFlipped = 0;

	// Check four neighbors: remember periodic BC!
	// If in same direction as original spin oldSpin
	// then with probability p flip it, call flipNeighbors

	int next = i+1;
	if (next == S->GetHeight())	next = 0;
	if (S->Get(next,j)==oldSpin && rand.uniform() < p)
	{
		S->Set(next,j,-oldSpin);
		numFlipped += FlipNeighbors(next,j)+1;
	}

	next = i-1;
	if (next == -1)	next = S->GetHeight()-1;
	if (S->Get(next,j)==oldSpin && rand.uniform() < p)
	{
		S->Set(next,j,-oldSpin);
		numFlipped += FlipNeighbors(next,j)+1;
	}

	next = j+1;
	if (next == S->GetWidth())	next = 0;
	if (S->Get(i,next)==oldSpin && rand.uniform() < p)
	{
		S->Set(i,next,-oldSpin);
		numFlipped += FlipNeighbors(i,next)+1;
	}

	next = j-1;
	if (next == -1)	next = S->GetWidth()-1;
	if (S->Get(i,next)==oldSpin && rand.uniform() < p)
	{
		S->Set(i,next,-oldSpin);
		numFlipped += FlipNeighbors(i,next)+1;
	}

	return numFlipped;
}
