// LMCDoc.cpp : implementation of the CLMCDoc class
//

#include "stdafx.h"
#include "LMC.h"

#include "LMCDoc.h"
#include "ParameterDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLMCDoc

IMPLEMENT_DYNCREATE(CLMCDoc, CDocument)

BEGIN_MESSAGE_MAP(CLMCDoc, CDocument)
	//{{AFX_MSG_MAP(CLMCDoc)
	ON_COMMAND(IDD_PARAM_CHANGE, OnParamChange)
	ON_COMMAND(ID_PARAMETER_CHANGE, OnParamChange)
	ON_COMMAND(ID_MAG_RESET, OnMagReset)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLMCDoc construction/destruction

CLMCDoc::CLMCDoc()
{
	sim = new IsingSimulation();
	sim->Attach(&magnetizationObserver);
	sim->Attach(&correlationObserver);	// XXX Probably a bad idea
	sweepsPerDraw = 1;
	pauseThread = false;
	oldAlgorithm = 2;
}

CLMCDoc::~CLMCDoc()
{
	delete sim;
}

BOOL CLMCDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CLMCDoc serialization

void CLMCDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CLMCDoc diagnostics

#ifdef _DEBUG
void CLMCDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CLMCDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLMCDoc commands

void CLMCDoc::OnParamChange() 
{
	CParameterDialog dialogBox;
	
	dialogBox.m_height = sim->GetHeight();
	dialogBox.m_width = sim->GetWidth();
	dialogBox.m_temperature = sim->GetTemperature();
	dialogBox.m_sweeps_per_draw = GetSweepsPerDraw();
	dialogBox.m_magnetic_field = sim->GetMagneticField();
	dialogBox.m_algorithm = oldAlgorithm;
	dialogBox.m_upRadius = 0.0;	// If not zero, set all spins down except circle radius upRadius

	if (dialogBox.DoModal()==IDOK)
	{
		pauseThread = true;
		threadPaused.Lock();	// Wait for thread to pause
		threadPaused.Unlock();	// Silly Microsoft convention

		if (dialogBox.m_height!=sim->GetHeight() || 
			dialogBox.m_width!=sim->GetWidth())
			sim->SetHeightWidth(dialogBox.m_height,dialogBox.m_width);
		if (dialogBox.m_temperature!=sim->GetTemperature())
			sim->SetTemperature(dialogBox.m_temperature);
		if (dialogBox.m_sweeps_per_draw!=GetSweepsPerDraw())
			SetSweepsPerDraw(dialogBox.m_sweeps_per_draw);
		if (dialogBox.m_magnetic_field!=sim->GetMagneticField())
			sim->SetMagneticField(dialogBox.m_magnetic_field);
		if (dialogBox.m_algorithm!=oldAlgorithm) {
			SpinDynamics* oldDynamics;
			switch(dialogBox.m_algorithm) {
			case 0:
				oldDynamics = sim->SwitchDynamics(new SpinDynamics(sim->GetLattice()));
				break;
			case 1:
				oldDynamics = sim->SwitchDynamics(new WolffDynamics(sim->GetLattice()));
				break;
			case 2:
				oldDynamics = sim->SwitchDynamics(new ContinuousTime(sim->GetLattice()));
			}
			delete oldDynamics;
			oldAlgorithm=dialogBox.m_algorithm;
		}
		if (dialogBox.m_upRadius != 0.0)
		{
			sim->GetDynamics()->SetRadius(dialogBox.m_upRadius);
		}
		sim->SetTime(0.0);	// Reset time to zero on param change
		paramsChanged.SetEvent();	// Restart thread
	}
}

void CLMCDoc::OnMagReset() 
{
	magnetizationObserver.Reset();
}
