// LMCView.cpp : implementation of the CLMCView class
//

#include "stdafx.h"
#include "LMC.h"

#include "LMCDoc.h"
#include "LMCView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLMCView

IMPLEMENT_DYNCREATE(CLMCView, CView)

BEGIN_MESSAGE_MAP(CLMCView, CView)
	//{{AFX_MSG_MAP(CLMCView)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

UINT run_thread(LPVOID pview)
{
	CLMCView *view = (CLMCView *)pview;

	CLMCDoc *pDoc = view->GetDocument();
	IsingSimulation *sim = pDoc->sim;
	sim->Attach(view);

	while (!view->stopThread)
	{
		sim->Notify();
		for(int sweep=0;sweep<pDoc->GetSweepsPerDraw() && !view->stopThread;sweep++)
		{
			sim->Sweep();
			Sleep(0);
		}
		if (pDoc->pauseThread)
		{
			pDoc->threadPaused.SetEvent();	// Acknowledge thread pause
			pDoc->paramsChanged.Lock();		// Wait for parameters to change
			pDoc->paramsChanged.Unlock();	// Silly Microsoft convention
			pDoc->pauseThread=false;
		}
	}

	view->stopThread=false;
	view->threadDone.SetEvent();

	return 1;
}

/////////////////////////////////////////////////////////////////////////////
// CLMCView construction/destruction

CLMCView::CLMCView()
{
	stopThread = false;
}

CLMCView::~CLMCView()
{
	if (!stopThread)		// Thread still running?
	{
		stopThread=TRUE;	// Ask thread to stop.
		threadDone.Lock();	// Wait for thread to announce exit with SetEvent
		threadDone.Unlock();	// Meaningless Windows convention: analogy to semaphores according to Matt Kuntz
	}
}

BOOL CLMCView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CLMCView drawing

void CLMCView::OnDraw(CDC* pDC)
{
	CLMCDoc* pDoc = GetDocument();
//	ASSERT_VALID(pDoc);	// XXX Fails if not commented out

	pDC->SetMapMode(MM_ISOTROPIC);
	CRect windowRect;

	GetClientRect(windowRect);

	SpinLattice *lattice = pDoc->sim->GetLattice();
	pDC->SetWindowExt(lattice->GetWidth(), lattice->GetHeight());
	pDC->SetViewportExt(windowRect.Width(), -windowRect.Height());	// Make y-axis increase going up
	pDC->SetViewportOrg(0, windowRect.Height());	// Make 0,0 on LLHS

	CBrush black(RGB(0,0,0));
	CBrush red(RGB(255,0,0));
	RECT current;

	for (int i=0; i<lattice->GetHeight(); i++)
	{
		current.top = i+1;
		current.bottom = i;
		for (int j=0; j<lattice->GetWidth(); j++)
		{
			current.left = j;
			current.right = j+1;
			if (lattice->Get(i,j)==1) 	pDC->FillRect(&current,&red);
			else						pDC->FillRect(&current,&black);
		}
	}
	pDC->SetTextColor(RGB(255,255,255));
	pDC->SetBkMode(TRANSPARENT);

	sprintf(textOut,"Time = %5.2lf, Magnetization = % 3.4lf",
		pDoc->sim->GetTime(), 
		((double) pDoc->sim->GetLattice()->GetMagnetization())/(pDoc->sim->GetWidth()*pDoc->sim->GetHeight()));
	pDC->TextOut(0,lattice->GetHeight(),textOut);

	sprintf(textOut,"Mbar = % 3.4lf, Chi = % 3.4lf",
		pDoc->magnetizationObserver.Mbar(), 
		pDoc->magnetizationObserver.Susceptibility());
	pDC->TextOut(0,lattice->GetHeight()*0.9,textOut);

}

/////////////////////////////////////////////////////////////////////////////
// CLMCView printing

BOOL CLMCView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CLMCView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CLMCView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CLMCView diagnostics

#ifdef _DEBUG
void CLMCView::AssertValid() const
{
	CView::AssertValid();
}

void CLMCView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CLMCDoc* CLMCView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CLMCDoc)));
	return (CLMCDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLMCView message handlers


void CLMCView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
	AfxBeginThread(run_thread,this);	
}


void CLMCView::Update(Subject *sim)
{

	CClientDC dc(this);
	OnDraw(&dc);
}
